var PurchasesManager =
{
    username: null,
    ready: false,
    purchaseCompleteCallback: null,
    errorCallback: null,
    refreshCallback: null,
};


PurchasesManager.init = function() {
    if(!window.cordova || !window.store) return;

    window.store.error(PurchasesManager.onError);
    window.store.when('consumable').approved(function(p) {
        p.verify();
    });
    window.store.when('consumable').cancelled(function() {
        if(PurchasesManager.errorCallback) PurchasesManager.errorCallback();
    });
    window.store.when('consumable').verified(function(data) {
        console.log("Billing buy response: " + JSON.stringify(data));
        if(PurchasesManager.purchaseCompleteCallback) {
            PurchasesManager.purchaseCompleteCallback(data);
        } else {
            PurchasesManager.sendToServerFull(null, null, data,
                function(rsp){
                    data.finish();
                    console.log("Billing ITEM CONSUMED " + JSON.stringify(rsp));
                },
                function(err){ console.log("Billing ERROR CONSUMING ITEM " + JSON.stringify(err));
                });
        }

    });

    // empty validator to force fill appStoreReceipt
    window.store.validator = function(product, callback) {
        callback(true, {});
    };

    console.log('Billing: ready');
    PurchasesManager.ready = true;
};

PurchasesManager.onError = function(err) {
    if(PurchasesManager.errorCallback) PurchasesManager.errorCallback();
    console.log("Billing error: " + JSON.stringify(err));
};

PurchasesManager.refresh = function(tariffs, callback) {
    if(!PurchasesManager.ready) return console.log('Billing: not ready...');

    if (callback) {
        window.store.ready(function() {
            var ret = [];
            for(var j = 0; j < window.store.products.length; j++) {
                var p = window.store.products[j];
                if(p.id && p.price) {
                    for (var k = 0; k < tariffs.length; k++) {
                        if (tariffs[k].id === p.id) {
                            ret.push({"id": p.id,
                                "title":  ANDROID_MODE ? p.title : p.description,
                                "price": p.price,
                                "mandatory": tariffs[k].mandatory
                            });
                            break;
                        }
                    }
                }
            }
            callback(ret);
        });
    }
    if (!window.store.ready()) {
        var arr = [];
        for (var i = 0; i < tariffs.length; i++) {
            arr.push({
                id:     tariffs[i].id,
                type:   window.store.CONSUMABLE
            });
        }

        window.store.register(arr);
        window.store.refresh();
    }
};

PurchasesManager.purchase = function(productId, payload) {
    if(!PurchasesManager.ready) return;
    console.log('Billing buy: ' + productId + ' (payload: '+payload+')');
    window.store.order(productId, {
        developerPayload: payload
    });
};

PurchasesManager.sendToServerFull = function (receiver, server, data, okCallback, errCallback) {
    var method = null;
    var params = null;
    if (ANDROID_MODE) {
        var orderId = data.transaction.id || null;
        var receipt = JSON.parse(data.transaction.receipt);
        var purchaseTime = (receipt.purchaseTime || (new Date()).getTime()) + "";
        var purchaseToken = receipt.purchaseToken;

        method = "onGooglePaymentFull";
        params = [receiver, server, receipt.productId, orderId, purchaseTime, purchaseToken];
    } else { // IOS_MODE
        method = "onApplePayment";
        params = [receiver || PurchasesManager.username, server, data.transaction.appStoreReceipt];
    }

    console.log("SEND TO SERVER " + method + " " + JSON.stringify(params));
    PortalService.account.callMethod(method, params, function () {
        data.finish();
        if (typeof okCallback == 'function') {
            okCallback(data.transaction);
        }
    }, errCallback);
};

PurchasesManager.onUserLogin = function(username, server, tariffs) {
    PurchasesManager.username = username;
    if (tariffs) {
        console.log("RESTORE PURCHASES");
        PurchasesManager.refresh(tariffs, null);
    }
};
