function ArtifactTip()
{
	this.artifact = null; // FArtifact
	this.doesntWork = null; // Array
}

ArtifactTip.prototype.__construct = function()
{
	this.altarLabel = I18.f('SACRIFICIAL_ITEM');
}

ArtifactTip.prototype.artifact = null; // FArtifact
ArtifactTip.prototype.doesntWork = null; // Array

ArtifactTip.alert = function(artifact, doesntWork, forAltar, forUpgrade, init, actions)
{
	var tip = ArtifactTip.create(artifact, doesntWork, forAltar, forUpgrade, init);
	var caption = APP.isHeropedia ? 'ARTIFACT_NAME' : 'ARTIFACT_TYPE_' + artifact.declaration.type.name + '_NAME';
	if (artifact.declaration.type.name == ArtifactType.RUNE.name) caption = 'RUNE';
	if (artifact.declaration.type.name == ArtifactType.RECIPE.name) caption = 'RECIPE';

	var a = TipDialog.createWithActions(tip.DOM.outerHTML, I18.f(caption), actions);
	return a;
}

ArtifactTip.alertWithCost = function(artifact, doesntWork, estate, actions)
{
	var tip = ArtifactTip.create(artifact, doesntWork, null, null, null);
	
	var caption = I18.f('ARTIFACT_TYPE_' + artifact.declaration.type.name + '_NAME');
	if (artifact.declaration.type.name == ArtifactType.RUNE.name) caption = I18.f('RUNE');
	if (artifact.declaration.type.name == ArtifactType.RECIPE.name) caption = I18.f('RECIPE');

	var cost = COM.create('ArtifactCostView');
	cost.update(artifact, estate);
	tip.DOM.appendChild(cost.DOM);
	
	var a = TipDialog.createWithActions(tip.DOM.outerHTML, caption, actions);
	return a;
}

ArtifactTip.create = function(artifact, doesntWork, forAltar, forUpgrade, init)
{
	var artifactTip = COM.create('ArtifactTip');
	artifactTip.update(artifact, doesntWork, forAltar, forUpgrade);
	if (typeof init == 'function')
	{
		init(artifactTip);
	}
	return artifactTip;
}

ArtifactTip.prototype.update = function(artifact, doesntWork, forAltar, forUpgrade)
{
	this.artifact = artifact;
	var decl = artifact.declaration; // ArtifactDeclaration
	var type = decl.type; // ArtifactType
	
	COM(this.info).update(this.artifact);

	UI.show(artifact.tillTick > 0, [this.timerBox]);

	if (UI.show(decl.heroClasses != null, [this.forClassBlock])) {
		this.forClassLabel.innerHTML = I18.f('FOR_HERO_CLASSES', Strings.join(decl.heroClasses, ', ', I18.getHeroClassName));
	}

	this.doesntWork = doesntWork;
	
	if(UI.show(decl.parts && decl.parts.length > 0, [this.bodyPartsLabel]))
	{
		this.bodyPartsLabel.innerHTML = I18.f('INTENDED_FOR', decl.twoHanded 
			? I18.f('FOR_BOTH_HANDS') 
			: Strings.join(Arrays.map(decl.parts, I18.getForBodyPart), "/", null)
		);
	}
	
	var heropedia = APP.isHeropedia; //FlashVars.isHeropedia
	if (UI.show(!heropedia && decl.setName, [this.setBox])) 
	{
		this.setLabel.innerHTML = I18.f('SET') + " " + Texts.artifactSetName(decl.setName);
		var maxItemsCount = ArtifactDeclaration.maxSetItemsCount(decl.setPassive);
		COM(this.setPassiveMaxDesc).setTitleStyle("color: #B38600");
		COM(this.setPassiveMaxDesc).setTitleText(I18.f('SET_SKILLS_MAX'));
		if (artifact.setItemsCountCompatible() >= maxItemsCount) {
			UI.show(false, [this.setPassiveDesc, this.spacer]);
			if(UI.show(true, [this.passiveMaxBox])) {
				COM(this.setPassiveMaxDesc).update(FConfig.getPassive(decl.setPassive, maxItemsCount));
			}
			this.setBox.style.opacity = 1;
			this.passiveMaxBox.style.opacity = 1;
		} else {
			var actions = FConfig.getPassive(decl.setPassive, this.artifact.setItemsCountCompatible()); // ActionsDecl
			if(UI.show(actions != null, [this.setPassiveDesc, this.spacer])) {
				COM(this.setPassiveDesc).setTitleText(I18.f('SET_SKILLS', artifact.setItemsCountCompatible(), maxItemsCount));
				COM(this.setPassiveDesc).update(actions);
			}
			if(UI.show(true, [this.passiveMaxBox])) {
				COM(this.setPassiveMaxDesc).update(FConfig.getPassive(decl.setPassive, maxItemsCount), actions);
			}
			if (actions != null) {
				this.setBox.style.opacity = 1;
				this.passiveMaxBox.style.opacity = 0.5;
			} else {
				this.setBox.style.opacity = 0.5;
				this.passiveMaxBox.style.opacity = 1;
			}
		}
	}
	
	if (UI.show(decl.level > 0, [this.doesntWorkText]))
	{
	    COM(this.doesntWorkText).update(doesntWork);
	}
	
	this.passives.innerHTML = '';
	if (UI.show(decl.passives, [this.passives]))
	{
	    for (var i = 0; i <= this.artifact.upgradesCount; i++) 
	    {
	        this.addPassiveDesc(decl, i, decl.type.name == ArtifactType.RUNE.name ? I18.f("RUNE_SKILLS") : i == 0 ? null : I18.f("UPGRADE_SKILLS"), i > 0);
	    }
	    if (forUpgrade) 
	    {
            if (FConfig.ARTIFACTS_CONVERSION_ALLOWED && this.artifact.convertedToDeclaration)
            {
                this.addPassiveDesc(this.artifact.convertedToDeclaration, 0, I18.f("ANCIENT_SKILLS"));
            }
            this.addPassiveDesc(decl, this.artifact.upgradesCount + 1, I18.f("UPGRADE_SKILLS"), true);
	    }
	}

	// spell tooltip
	var spellDecl = decl.spell ? FConfig.getSpell(decl.spell, decl.level) : null; // SpellLevelDecl
	if (UI.show(spellDecl, [this.spellName, this.spellDesc]))
	{
		this.spellName.innerHTML = I18.f('SPELL_NAME') + " " + Texts.spellName(decl.spell);
		COM(this.spellDesc).showEffectsText = !decl.isBottle();
		COM(this.spellDesc).update(decl.spell, spellDecl, ArtifactType.SCROLL == decl.type, false, null, null, artifact.cooldownTillTick);
	}

	if (UI.show(this.artifact.runes.length, [this.runeBox]))
	{
		$(this.runeEffectContainer).html('');
		for (var i = 0, len = this.artifact.runes.length; i < len; i++)
		{
			var runePassive = this.artifact.runes.getItemAt(i);
			var action = FConfig.getPassive(runePassive, 1);
			EffectsView.createDescriptiveView(this.runeEffectContainer, action.dtoActions, false, true, false, true);
		}
	}

	if (UI.show(this.artifact.runeWord && this.artifact.runeWord.length, [this.runeWordBox]))
	{
		$(this.runeWordEffectContainer).html('');
		EffectsView.createDescriptiveView(this.runeWordEffectContainer, this.artifact.runeWord, false, true, false, true);
	}
	if(UI.show(decl.type.name == ArtifactType.RECIPE.name, [this.description])){
		$(this.description).html('');
		EffectsView.createRecipeView(this.description, decl);
	}

	UI.show(forAltar, [this.altarBox]);
}

ArtifactTip.prototype.addPassiveDesc = function(decl, index, title, colored)
{
	if (decl.passives.length <= index) return;

	var passive = FConfig.getPassive(decl.passives[index], decl.level); // ActionsDecl
	var desc = COM.create('PassiveDescription'); // PassiveDescription

	desc.titleText = title;
	desc.update(passive);

	if (colored)
	{
		desc.DOM.style.color = Effects.BLUE;
	}

	this.passives.appendChild(desc.DOM);
}