function RuneMasterUI()
{
    this.tabVisible = true;
    this.tabToolTip = I18.f("RUNE_MASTER");

    this.estate = null;
}

RuneMasterUI.prototype.__construct = function()
{
    COM.bindExclusiveClick(this.performRiteButton, $.proxy(this.onPerformRiteClick, this));
    COM.bindExclusiveClick(this.extractButton, $.proxy(this.onExtractRuneClick, this));
    COM.bindExclusiveClick(this.useRecipeButton, $.proxy(this.onUseRecipeClick, this));
    COM(this.pearlPrice).ownerUser = FUser.CURRENT_USER;
}

RuneMasterUI.prototype.update = function(estate, hero, parentDialog)
{
    this.estate = estate;

    var artifacts = this.estate.owner.runeMasterCube.artifacts;

    UI.makeNChilds(this.cubeTile, artifacts.length, this.slotFactory);
    for (var i=0; i<artifacts.length; i++)
    {
        var art = artifacts[i];
        var com = COM(this.cubeTile.childNodes[i]);
        com.update(this.estate, hero, this.estate.owner.runeMasterCube, i, art, [], parentDialog);
    }

    var artifact = this.findSingleArt(artifacts);
    if (artifact && artifact.runes.length > 0)
    {
        var price = FConfig.RUNE_EXTRACT_PRICE.getValueOrNull(artifact.runes.length);
        COM(this.pearlPrice).update(price);
        COM(this.extractButton).reason = this.estate.owner.pearl >= price ? null : I18.f("NOT_ENOUGH_PEARL");
    }
    else
    {
        COM(this.pearlPrice).update(0);
        COM(this.extractButton).reason = I18.f("EXTRACT_RUNE_TIP");
    }

    var recipe = this.findSingleRecipe(artifacts);
    if (recipe != null && FConfig.RECIPES.containsKey(recipe.declaration.name)) {
        var recipeDecl = FConfig.RECIPES.getValueOrNull(recipe.declaration.name);
        var price2 = recipeDecl.pearlPrice;
        COM(this.recipePrice).update(price2);
        COM(this.useRecipeButton).reason = this.estate.owner.pearl >= price2 ? null : I18.f("NOT_ENOUGH_PEARL");
    } else {
        COM(this.recipePrice).update(0);
        COM(this.useRecipeButton).reason = I18.f("USE_RECIPE_TIP");
    }

}

RuneMasterUI.prototype.onPerformRiteClick = function()
{
    Service.hero.callMethod("performRite", [this.estate.id], null, function (e) {
            if (e.__className__ == "HeroException")
            {
                Alert.create(e.getMessage());
                return true;
            }
            return false;
        }
    );
}

RuneMasterUI.prototype.onExtractRuneClick = function()
{
    Service.hero.callMethod("extractRunes", [this.estate.id], null, function (e) {
            if (e.__className__ == "HeroException")
            {
                Alert.create(e.getMessage());
                return true;
            }
            return false;
        }
    );
}

RuneMasterUI.prototype.onUseRecipeClick = function() {
    var recipe = this.findSingleRecipe(this.estate.owner.runeMasterCube.artifacts);
    var recipeDecl = recipe == null ? null : FConfig.RECIPES.getValueOrNull(recipe.declaration.name);
    if (recipeDecl == null) {
        Alert.create(I18.heroError("WRONG_RUNE_RECIPE"));
        return;
    }
    Service.hero.callMethod("useRecipe", [this.estate.id, recipeDecl.pearlPrice, recipeDecl.selectArtifacts == null || recipeDecl.selectArtifacts.length > 1 ? null : recipeDecl.selectArtifacts[0]], null, $.proxy(function(e) {
        if (e.__className__ == "HeroException") {
            Alert.create(e.getMessage(), I18.f('ERROR'));
            return true;
        }
        if (e.__className__ == "TemplateException" && e.type == "USER_INPUT_REQUIRED") {
            var dialog = COM.create("UseTournamentServiceDialog");
            var reward = new FQuestReward();
            reward.rewardType = "REWARD_SELECT_ARTIFACT";
            reward.rewardParams = new FMap();
            reward.rewardParams.put("artifacts", e.params.getValueOrNull("data"));
            reward.rewardParams.put("perfect", false);
            dialog.reward = [reward];
            var estateId = this.estate.id;
            dialog.onChoose = $.proxy(function (rewardData) {
                Service.hero.callMethod("useRecipe", [estateId, recipeDecl.pearlPrice, rewardData[0]], null, function (e) {
                    if (e.__className__ == "HeroException") {
                        Alert.create(e.getMessage(), I18.f('ERROR'));
                        return true;
                    }
                    return false;
                });
                return true;
            });
            dialog.popup();
            return true;
        }
        return false;
    }, this));
};

RuneMasterUI.prototype.findSingleArt = function(artifacts)
{
    var result = null;
    for(var i=0; i<artifacts.length; i++)
    {
        var art = artifacts[i];
        if (art)
        {
            if (result) return null;
            result = art;
        }
    }

    return result;
}

RuneMasterUI.prototype.findSingleRecipe = function(artifacts)
{
    var result = null;
    for(var i=0; i<artifacts.length; i++)
    {
        var art = artifacts[i];
        if (art && art.declaration != null && art.declaration.type.name == ArtifactType.RECIPE.name)
        {
            if (result) return null;
            result = art;
        }
    }
    return result;
}

RuneMasterUI.prototype.slotFactory = function()
{
    var slot = COM.create('ArtifactSlotView');
    slot.isCube = true;
    return slot;
}

RuneMasterUI.prototype.tick = function(time)
{
}
