function ArtifactDeclaration(name)
{
	if (typeof name == 'undefined') name = '';
	
	this.name = name;
	this.pic = name;
	this.parts = name ? [] : null;
	this.twoHanded = false;
    this.heroClasses = null;
	this.level = 0;
	this.spell = '';
	this.passives = '';
	this.type = name ? ArtifactType.ARTIFACT : null; // ArtifactType
	this.cost = name ? new ArtifactCost() : null; // ArtifactCost
	this.buyOutCost = name ? new ArtifactCost() : null; // ArtifactCost
	this.convertedTo = '';
	if (this.cost)
	{
		this.cost.resource = new Funds();
	}
	if (this.buyOutCost)
	{
		this.buyOutCost.resource = new Funds();
	}
	this.setName = '';
	this.setPassive = '';
	this.setLevel = 0;
}

ArtifactDeclaration.prototype.__CLS__ = 'game.model.heroes.artifacts.ArtifactDeclaration';
ArtifactDeclaration.prototype.name = '';
ArtifactDeclaration.prototype.pic = '';
ArtifactDeclaration.prototype.parts = null;
ArtifactDeclaration.prototype.twoHanded = false;
ArtifactDeclaration.prototype.heroClasses = null;
ArtifactDeclaration.prototype.level = 0;
ArtifactDeclaration.prototype.spell = '';
ArtifactDeclaration.prototype.passives = '';
ArtifactDeclaration.prototype.type = null; // ArtifactType
ArtifactDeclaration.prototype.cost = null; // ArtifactCost
ArtifactDeclaration.prototype.buyOutCost = null; // ArtifactCost
ArtifactDeclaration.prototype.convertedTo = '';
ArtifactDeclaration.prototype.setName = '';
ArtifactDeclaration.prototype.setPassive = '';
ArtifactDeclaration.prototype.setLevel = 0;

ArtifactDeclaration.UNKNOWN_ARTIFACT = new ArtifactDeclaration("UNDEFINED_ITEM");

ArtifactDeclaration.unknown = function(type, level)
{
    var decl = new ArtifactDeclaration("UNKNOWN_" + type + "_" + level);
    decl.name = "UNKNOWN";
    decl.pic = "UNKNOWN_" + type + "_" + level;
    decl.spell = "UNKNOWN";
    decl.type = type;
    decl.level = level;
    return decl;
}

ArtifactDeclaration.prototype.getSpell = function()
{
    return new FSpell(this.spell, this.level, this);
}

ArtifactDeclaration.prototype.fitsIn = function(part)
{
    if (!this.parts) return false;
    for (var i = 0; i < this.parts.length; i++)
    {
    	if (this.parts[i].name == part.name) return true;
    }
    return false;
}

ArtifactDeclaration.prototype.givesSpell = function()
{
    return Boolean(this.spell);
}

ArtifactDeclaration.prototype.givesPassive = function()
{
    return Boolean(this.passives);
}

ArtifactDeclaration.prototype.isBottle = function()
{
    for (var i = 0; i < ArtifactType.BOTTLES.length; i++)
    {
    	if (ArtifactType.BOTTLES[i].name == this.type.name) return true;
    }
    return false;
}

ArtifactDeclaration.prototype.isScrollOrBottle = function()
{
    return ArtifactType.SCROLL.name === this.type.name || this.isBottle();
}

ArtifactDeclaration.prototype.isStackable = function()
{
    for (var i = 0; i < ArtifactType.STACKABLE.length; i++)
    {
    	if (ArtifactType.STACKABLE[i].name == this.type.name) return true;
    }
    return false;
}

ArtifactDeclaration.maxSetItemsCount = function(passive) {
    var passiveDecl = FConfig.passives.getValueOrNull(passive);
    if (passiveDecl == null) {
        return 0;
    }
    var max = 0;
    var keys = passiveDecl.keys;
    for (var i = 0; i < keys.length; i++) {
        max = Math.max(max, keys[i]);
    }
    return max;
};
