function ServiceGridRow()
{
	this.decl = null;
    this.user = null;
    this.estate = null;
    this.onBuyService = null;
    this.onAutoOrderService = null;

    this.combo = null;
    this.comboChanged = false;

    this.__defineGetter__('selectedItemIndex', function()
    {
        return this.combo != null ? COM(this.combo).selectedIndex : 0;
    });
    this.__defineGetter__('selectedItem', function()
    {
        return this.decl.items.getItemAt(this.selectedItemIndex);
    });

}

ServiceGridRow.prototype.__construct = function()
{
	COM.bindExclusiveClick(this.buy, $.proxy(this.doOrder, this));
	COM.bindExclusiveClick(this.autoOrderCheckbox, $.proxy(this.doAutoOrder, this));
}

ServiceGridRow.prototype.init = function(user, estate, decl, onBuyService, onAutoOrderService)
{
    this.decl = decl;
    this.user = user;
    this.estate = estate;
    this.onBuyService = onBuyService;
    this.onAutoOrderService = onAutoOrderService;

    var comboItems = decl.getComboItems();
    if (comboItems.length == 1)
    {
        this.comboItem.appendChild(UI.label(comboItems[0]));
    }
    else
    {
        this.combo = COM.create("ComboBox");
        this.combo.dataProvider = comboItems;
        this.combo.id = "combo";
        this.combo.selectedIndex = 0;
        $(this.combo.DOM).on("change", $.proxy(function()
        {
            this.comboChanged = true;
            this.updateDescription();
            this.tickImpl();
        }, this));
        this.comboItem.appendChild(this.combo.DOM);
    }
    this.updateDescription();

    if (decl.service == ServiceDecl.VACATION)
    {
        this.descBox.style.textDecoration = "underline";
        COM.bindExclusiveClick(this.descBox, function()
        {
        	Alert.create(I18.getMegaString(I18.f("VACATION_TIP"), FConfig));
        });
    }
	
    this.tickImpl();
}

ServiceGridRow.prototype.doOrder = function()
{
    this.onBuyService(this.decl.service, this.selectedItemIndex, null,
        this.estate ? this.estate.id : GlobalSelectionCitiesCombo.selectedCityId,
        null, COM(this.buy).caption == I18.f("PAYMENT_PROLONGATE"));
}

ServiceGridRow.prototype.doAutoOrder = function()
{
    this.onAutoOrderService(this.decl.service, COM(this.autoOrderCheckbox).checked, this.selectedItemIndex,
        this.decl.items.length > 1 ? this.decl.getComboItem(this.selectedItemIndex) : null,
        !this.estate ? GlobalSelectionCitiesCombo.selectedCityId : this.estate.id);
}

ServiceGridRow.prototype.updateStatus = function(text, on, warning)
{
    UI.show(true, [this.statusLabel], $.proxy(function()
    {
        this.statusLabel.innerHTML = text;
        PaymentDialog.updateStateLabel(this.statusLabel, warning);
    }, this));
}

ServiceGridRow.prototype.updateDescription = function () {
    this.descBox.innerHTML = '';
    var description = this.decl.getDescription(this.selectedItemIndex);
    if (description != null) {
        UIh.addItem(this.descBox, description);
    } else {
        EffectsView.createDescriptiveView(this.descBox, this.selectedItem.dtoReward, false, false, false, false);
    }
}

ServiceGridRow.prototype.tick = function(user, estate)
{
    this.estate = estate;
    this.user = user;
    this.tickImpl();
}

ServiceGridRow.prototype.tickImpl = function()
{
    var servicesCooldowns;
    var servicesActiveTill;
    var servicesValues;
    var servicesOrder;
    var autoOrder;
    var underAttack;
    
    if (this.estate)
    {
        servicesCooldowns = new FMap();
        servicesActiveTill = this.estate.getServicesActiveTill();
        servicesValues = this.estate.servicesValues;
        autoOrder = this.estate.servicesAutoOrder;
        underAttack = this.estate.isEstateUnderSiege;
        servicesOrder = this.estate.servicesOrder;
    }
    else
    {
        servicesCooldowns = this.user.servicesCooldowns;
        servicesActiveTill = this.user.getServicesActiveTill();
        servicesValues = this.user.servicesValues;
        autoOrder = this.user.servicesAutoOrder;
        underAttack = this.user.isUserUnderSiege;
        servicesOrder = this.user.servicesOrder;
    }

    COM(this.price).ownerUser = this.user;
    COM(this.price).update(this.selectedItem.price);
    
    var buy = COM(this.buy);
    
    var cooldown = servicesCooldowns.getValueOrNull(this.decl.service);
    if (!cooldown)
    {
        cooldown = 0;
    }
    var activeTill = servicesActiveTill.getValueOrNull(this.decl.service);
    if (!activeTill)
    {
        activeTill = 0;
    }
    var serviceValue = servicesValues.getValueOrNull(this.decl.service);
    if (!serviceValue)
    {
        serviceValue = 0;
    }

    var lastOrderedLevel = servicesOrder.getValueOrDefault(this.decl.service, -1);
    if (this.combo != null && !this.comboChanged && lastOrderedLevel >= 0 && lastOrderedLevel < this.decl.items.length && activeTill >= Tick.now()) {
        this.combo.selectedIndex = lastOrderedLevel;
    }

    if (activeTill >= Tick.now())
    {
        buy.caption = I18.f("PAYMENT_PROLONGATE");
        this.updateStatus(
            lastOrderedLevel >= 0 && lastOrderedLevel < this.decl.items.length ? I18.f("PAYMENT_TILL_VALUE", this.decl.getComboItem(lastOrderedLevel), Tick.toTimeFuture(activeTill)) : I18.f("PAYMENT_TILL", Tick.toTimeFuture(activeTill)),
                true,
                PaymentDialog.hasWarning(this.user, !this.estate ? null : this.estate.id, this.decl.service)
        );
    }
    else
    {
        buy.caption = I18.f("ORDER_SERVICE");
        this.updateStatus(
                (cooldown > Tick.now()) ? I18.mega("UNAVAILABLE_TILL", [cooldown]) :
                        (this.selectedItem.valueInfo.length != 0) ? I18.getString(this.selectedItem.valueInfo, [serviceValue]) : null,
                false
        );
    }
    
    var canBuy = false;
    var canAutoOrder = true;

    if (Tick.now() + FConfig.PEARL_MAX_ADVANCE_DURATION < activeTill)
    {
        buy.reason = I18.f("PEARL_PAYMENT_REACHED_MAX_ADVANCE");
    }
    else if (this.selectedItem.requirePositiveValue && serviceValue <= 0)
    {
        var tips = this.selectedItem.wrongValueTips.split(",");
        var index = -serviceValue;
        if (index >= tips.length)
        {
            index = tips.length - 1;
        }
        
        buy.reason = I18.getMegaString(I18.getString(tips[index]), this.decl.getRewardParams(this.selectedItemIndex));
    }
    else if (cooldown > Tick.now())
    {
        buy.reason = I18.f("PEARL_BOOST_ERROR_COOLDOWN");
    }
    else if (!this.selectedItem.underAttack && underAttack)
    {
        buy.reason = I18.getMegaString(I18.f("PEARL_BOOST_ERROR_UNDER_ATTACK"), FConfig);
    }
    else if (this.user.pearl < this.selectedItem.price)
    {
        buy.reason = I18.f("NOT_ENOUGH_PEARL");
    }
    else if (activeTill >= Tick.now() && lastOrderedLevel >= 0) {
        if (lastOrderedLevel != this.selectedItemIndex) {
            buy.caption = I18.f("ORDER_SERVICE");
            canAutoOrder = false;
        }
        if (lastOrderedLevel > this.selectedItemIndex) {
            buy.reason = I18.f("PROLONGATE_SAME_VALUE");
        } else {
            canBuy = true;
            buy.reason = "";
        }
    }
    else
    {
        canBuy = true;
        buy.reason = "";
    }
    
    var manageable = this.user.username == FUser.CURRENT_USER.username;
    buy.enabled = manageable;
    
    var cb = COM(this.autoOrderCheckbox);
    
    cb.DOM.style.display = this.selectedItem.autoOrderable ? "" : "none";
    cb.disabled = !((activeTill >= Tick.now() || autoOrder.containsKey(this.decl.service)) && manageable && canAutoOrder);
    //cb.toolTip = I18.AUTO_ORDER.f() + (autoOrderCheckbox.enabled ? "" : "\n" + I18.NOT_ORDERED.f());
    cb.checked = autoOrder.containsKey(this.decl.service) && lastOrderedLevel == this.selectedItemIndex;
}